@@ -8,25 +8,25 @@ from pyspark.sql import functions as F
 from pyspark.sql.types import *
 from delta.tables import DeltaTable
 
+requiredColumns = {
+    "DienstId": IntegerType(),
+    "Source": StringType(),
+    "SurrogateKey": LongType(),
+    "FacturatieMaand": StringType(),
+    "VerkooprelatieId": StringType(),
+    "ItemCode": StringType(),
+    "Aantal": FloatType()
+}
+
+optionalColumns = {
+    "AfwijkendePrijs": FloatType(),
+    "DatumVanOrigineel": StringType(),
+    "DatumTotOrigineel": StringType(),
+    "AantalOrigineel": FloatType(),
+    "DurationOrigineel": FloatType()
+}
+
 def validateItemsSchema(items):
-    requiredColumns = {
-        "DienstId": IntegerType(),
-        "Source": StringType(),
-        "SurrogateKey": LongType(),
-        "FacturatieMaand": StringType(),
-        "VerkooprelatieId": StringType(),
-        "ItemCode": StringType(),
-        "Aantal": FloatType()
-    }
-
-    optionalColumns = {
-        "AfwijkendePrijs": FloatType(),
-        "DatumVanOrigineel": StringType(),
-        "DatumTotOrigineel": StringType(),
-        "AantalOrigineel": FloatType(),
-        "DurationOrigineel": FloatType()
-    }
-
     schemaDict = {f.name: f.dataType for f in items.schema.fields}
 
     for columnName, expectedType in requiredColumns.items():
@@ -62,22 +62,50 @@ def writeLines(items):
 
     facturatieMaand = items.select("FacturatieMaand").head()["FacturatieMaand"]
 
-    aggregated = items.groupBy(
+    existingOptionalColumns = [col for col in optionalColumns.keys() if col in items.columns]
+
+    if existingOptionalColumns:
+        filterExpression = " AND ".join([f"{col} IS NULL" for col in existingOptionalColumns])
+        itemsToAggregate = items.filter(filterExpression)
+        itemsToPreserve = items.filter(f"NOT ({filterExpression})")
+    else:
+        itemsToAggregate = items
+        itemsToPreserve = items.limit(0)
+
+    for col, colType in optionalColumns.items():
+        if col not in items.columns:
+            itemsToAggregate = itemsToAggregate.withColumn(col, F.lit(None).cast(colType))
+            itemsToPreserve = itemsToPreserve.withColumn(col, F.lit(None).cast(colType))
+
+    aggregated = itemsToAggregate.groupBy(
         "DienstId", "FacturatieMaand", "VerkooprelatieId", "ItemCode"
     ).agg(
         F.sum("Aantal").alias("Aantal"),
         F.collect_list("SurrogateKey").alias("SurrogateKeys")
-    ).withColumn(
+    )
+
+    for col, colType in optionalColumns.items():
+        aggregated = aggregated.withColumn(col, F.lit(None).cast(colType))
+
+    aggregated = aggregated.withColumn(
         "HasItems",
         F.when(
-            (F.size("SurrogateKeys") > 1) & (F.expr("aggregate(SurrogateKeys, true, (accumulator, x) -> accumulator AND x IS NOT NULL)")),
+            (F.size("SurrogateKeys") > 1)
+            & (F.expr("aggregate(SurrogateKeys, true, (accumulator, x) -> accumulator AND x IS NOT NULL)")),
             True
         ).otherwise(False)
     ).drop("SurrogateKeys")
 
+    itemsToPreserve = itemsToPreserve.withColumn(
+        "HasItems",
+        F.when(F.col("SurrogateKey").isNotNull(), True).otherwise(False)
+    )
+
+    lines = aggregated.unionByName(itemsToPreserve)
+
     if DeltaTable.isDeltaTable(tdsbrondata._spark, tablePath):
         deltaTable = DeltaTable.forPath(tdsbrondata._spark, tablePath)
         deltaTable.delete(f"FacturatieMaand = '{facturatieMaand}'")
-        aggregated.write.format("delta").mode("append").save(tablePath)
+        lines.write.format("delta").mode("append").save(tablePath)
     else:
-        aggregated.write.format("delta").mode("overwrite").option("overwriteSchema", "true").save(tablePath)
\ No newline at end of file
+        lines.write.format("delta").mode("overwrite").option("overwriteSchema", "true").save(tablePath)
\ No newline at end of file
